@@ -16,6 +16,8 @@
  */
 package org.apache.logging.log4j.core;
 
+import static org.apache.logging.log4j.core.util.ShutdownCallbackRegistry.SHUTDOWN_HOOK_MARKER;
+
 import java.beans.PropertyChangeEvent;
 import java.beans.PropertyChangeListener;
 import java.io.File;
@@ -24,6 +26,11 @@ import java.util.Collection;
 import java.util.Objects;
 import java.util.concurrent.ConcurrentMap;
 import java.util.concurrent.CopyOnWriteArrayList;
+import java.util.concurrent.ExecutorService;
+import java.util.concurrent.Executors;
+import java.util.concurrent.Future;
+import java.util.concurrent.RejectedExecutionException;
+import java.util.concurrent.TimeUnit;
 import java.util.concurrent.locks.Lock;
 import java.util.concurrent.locks.ReentrantLock;
 
@@ -38,6 +45,8 @@ import org.apache.logging.log4j.core.config.Reconfigurable;
 import org.apache.logging.log4j.core.impl.Log4jLogEvent;
 import org.apache.logging.log4j.core.jmx.Server;
 import org.apache.logging.log4j.core.util.Cancellable;
+import org.apache.logging.log4j.core.util.ExecutorServices;
+import org.apache.logging.log4j.core.util.Log4jThreadFactory;
 import org.apache.logging.log4j.core.util.NetUtils;
 import org.apache.logging.log4j.core.util.ShutdownCallbackRegistry;
 import org.apache.logging.log4j.message.MessageFactory;
@@ -47,15 +56,13 @@ import org.apache.logging.log4j.spi.LoggerRegistry;
 import org.apache.logging.log4j.spi.Terminable;
 import org.apache.logging.log4j.util.PropertiesUtil;
 
-import static org.apache.logging.log4j.core.util.ShutdownCallbackRegistry.*;
-
 /**
  * The LoggerContext is the anchor for the logging system. It maintains a list of all the loggers requested by
  * applications and a reference to the Configuration. The Configuration will contain the configured loggers, appenders,
  * filters, etc and will be atomically updated whenever a reconfigure occurs.
  */
-public class LoggerContext extends AbstractLifeCycle implements org.apache.logging.log4j.spi.LoggerContext, Terminable,
-        ConfigurationListener {
+public class LoggerContext extends AbstractLifeCycle
+        implements org.apache.logging.log4j.spi.LoggerContext, AutoCloseable, Terminable, ConfigurationListener {
 
     /**
      * Property name of the property change event fired if the configuration is changed.
@@ -72,6 +79,8 @@ public class LoggerContext extends AbstractLifeCycle implements org.apache.loggi
      * reference is updated.
      */
     private volatile Configuration configuration = new DefaultConfiguration();
+    private ExecutorService executorService;
+    private ExecutorService executorServiceDeamons;
     private Object externalContext;
     private String contextName;
     private volatile URI configLocation;
@@ -259,6 +268,7 @@ public class LoggerContext extends AbstractLifeCycle implements org.apache.loggi
                     this.shutdownCallback = ((ShutdownCallbackRegistry) factory).addShutdownCallback(new Runnable() {
                         @Override
                         public void run() {
+                            @SuppressWarnings("resource")
                             final LoggerContext context = LoggerContext.this;
                             LOGGER.debug(SHUTDOWN_HOOK_MARKER, "Stopping LoggerContext[name={}, {}]",
                                     context.getName(), context);
@@ -281,25 +291,33 @@ public class LoggerContext extends AbstractLifeCycle implements org.apache.loggi
         }
     }
 
+    @Override
+    public void close() {
+        stop();
+    }
+
     @Override
     public void terminate() {
         stop();
     }
 
     @Override
-    public void stop() {
+    public boolean stop(final long timeout, final TimeUnit timeUnit) {
         LOGGER.debug("Stopping LoggerContext[name={}, {}]...", getName(), this);
         configLock.lock();
+        final boolean shutdownEs;
+        final boolean shutdownEsd;
         try {
             if (this.isStopped()) {
-                return;
+                return true;
             }
 
             this.setStopping();
             try {
                 Server.unregisterLoggerContext(getName()); // LOG4J2-406, LOG4J2-500
-            } catch (final Exception ex) {
-                LOGGER.error("Unable to unregister MBeans", ex);
+            } catch (final LinkageError | Exception e) {
+                // LOG4J2-1506 Hello Android, GAE
+                LOGGER.error("Unable to unregister MBeans", e);
             }
             if (shutdownCallback != null) {
                 shutdownCallback.cancel();
@@ -308,14 +326,23 @@ public class LoggerContext extends AbstractLifeCycle implements org.apache.loggi
             final Configuration prev = configuration;
             configuration = NULL_CONFIGURATION;
             updateLoggers();
-            prev.stop();
+            if (prev instanceof LifeCycle2) {
+                ((LifeCycle2) prev).stop(timeout, timeUnit);
+            } else {
+                prev.stop();
+            }
             externalContext = null;
             LogManager.getFactory().removeContext(this);
-            this.setStopped();
+            final String source = "LoggerContext \'" + getName() + "\'";
+            shutdownEs = ExecutorServices.shutdown(executorService, timeout, timeUnit, source);
+            // Do not wait for daemon threads
+            shutdownEsd = ExecutorServices.shutdown(executorServiceDeamons, -1, timeUnit, source);
         } finally {
             configLock.unlock();
+            this.setStopped();
         }
         LOGGER.debug("Stopped LoggerContext[name={}, {}]...", getName(), this);
+        return shutdownEs && shutdownEsd;
     }
 
     /**
@@ -479,11 +506,18 @@ public class LoggerContext extends AbstractLifeCycle implements org.apache.loggi
      * @return The previous Configuration.
      */
     private Configuration setConfiguration(final Configuration config) {
-        Objects.requireNonNull(config, "No Configuration was provided");
+        if (config == null) {
+            LOGGER.error("No configuration found for context '{}'.", contextName);
+            // No change, return the current configuration.
+            return this.configuration;
+        }
         configLock.lock();
         try {
             final Configuration prev = this.configuration;
             config.addListener(this);
+            executorService = Executors.newCachedThreadPool(Log4jThreadFactory.createThreadFactory(contextName));
+            executorServiceDeamons = Executors.newCachedThreadPool(Log4jThreadFactory.createDaemonThreadFactory(contextName));
+
             final ConcurrentMap<String, String> map = config.getComponent(Configuration.CONTEXT_PROPERTIES);
 
             try { // LOG4J2-719 network access may throw android.os.NetworkOnMainThreadException
@@ -505,9 +539,9 @@ public class LoggerContext extends AbstractLifeCycle implements org.apache.loggi
 
             try {
                 Server.reregisterMBeansAfterReconfigure();
-            } catch (final Throwable t) {
+            } catch (final LinkageError | Exception e) {
                 // LOG4J2-716: Android has no java.lang.management
-                LOGGER.error("Could not reconfigure JMX", t);
+                LOGGER.error("Could not reconfigure JMX", e);
             }
             // AsyncLoggers update their nanoClock when the configuration changes
             Log4jLogEvent.setNanoClock(configuration.getNanoClock());
@@ -562,15 +596,19 @@ public class LoggerContext extends AbstractLifeCycle implements org.apache.loggi
         final ClassLoader cl = ClassLoader.class.isInstance(externalContext) ? (ClassLoader) externalContext : null;
         LOGGER.debug("Reconfiguration started for context[name={}] at URI {} ({}) with optional ClassLoader: {}",
                 contextName, configURI, this, cl);
-        final Configuration instance = ConfigurationFactory.getInstance().getConfiguration(contextName, configURI, cl);
-        setConfiguration(instance);
-        /*
-         * instance.start(); Configuration old = setConfiguration(instance); updateLoggers(); if (old != null) {
-         * old.stop(); }
-         */
-        final String location = configuration == null ? "?" : String.valueOf(configuration.getConfigurationSource());
-        LOGGER.debug("Reconfiguration complete for context[name={}] at URI {} ({}) with optional ClassLoader: {}",
-                contextName, location, this, cl);
+        final Configuration instance = ConfigurationFactory.getInstance().getConfiguration(this, contextName, configURI, cl);
+        if (instance == null) {
+            LOGGER.error("Reconfiguration failed: No configuration found for '{}' at '{}' in '{}'", contextName, configURI, cl);
+        } else {
+            setConfiguration(instance);
+            /*
+             * instance.start(); Configuration old = setConfiguration(instance); updateLoggers(); if (old != null) {
+             * old.stop(); }
+             */
+            final String location = configuration == null ? "?" : String.valueOf(configuration.getConfigurationSource());
+            LOGGER.debug("Reconfiguration complete for context[name={}] at URI {} ({}) with optional ClassLoader: {}",
+                    contextName, location, this, cl);
+        }
     }
 
     /**
@@ -624,4 +662,52 @@ public class LoggerContext extends AbstractLifeCycle implements org.apache.loggi
         return new Logger(ctx, name, messageFactory);
     }
 
+    /**
+     * Gets the executor service to submit normal tasks.
+     *
+     * @return the ExecutorService to submit normal tasks.
+     */
+    public ExecutorService getExecutorService() {
+        return executorService;
+    }
+
+    /**
+     * Gets the executor service to submit daemon tasks.
+     *
+     * @return the ExecutorService to submit normal daemon tasks.
+     */
+    public ExecutorService getExecutorServiceDeamons() {
+        return executorServiceDeamons;
+    }
+
+    /**
+     * Submits a Runnable task for normal execution and returns a Future representing that task. The Future's
+     * {@code get} method will return {@code null} upon <em>successful</em> completion.
+     *
+     * @param task the task to submit
+     * @return a Future representing pending completion of the task
+     * @throws RejectedExecutionException if the task cannot be
+     *         scheduled for execution
+     * @throws NullPointerException if the task is null
+     */
+    public Future<?> submit(final Runnable task) {
+        return executorService.submit(task);
+    }
+
+    /**
+     * Submits a Runnable task for daemon execution and returns a Future representing that task. The Future's
+     * {@code get} method will return {@code null} upon <em>successful</em> completion.
+     *
+     * @param task
+     *            the task to submit
+     * @return a Future representing pending completion of the task
+     * @throws RejectedExecutionException
+     *             if the task cannot be scheduled for execution
+     * @throws NullPointerException
+     *             if the task is null
+     */
+    public Future<?> submitDaemon(final Runnable task) {
+        return executorServiceDeamons.submit(task);
+    }
+
 }
