@@ -16,11 +16,19 @@
  */
 package org.apache.logging.log4j.core.config.builder.impl;
 
+import java.io.IOException;
+import java.io.OutputStream;
+import java.io.StringWriter;
 import java.lang.reflect.Constructor;
 import java.util.List;
+import java.util.Map;
+import javax.xml.stream.XMLOutputFactory;
+import javax.xml.stream.XMLStreamException;
+import javax.xml.stream.XMLStreamWriter;
 
 import org.apache.logging.log4j.Level;
 import org.apache.logging.log4j.core.Filter;
+import org.apache.logging.log4j.core.LoggerContext;
 import org.apache.logging.log4j.core.config.Configuration;
 import org.apache.logging.log4j.core.config.ConfigurationException;
 import org.apache.logging.log4j.core.config.ConfigurationSource;
@@ -37,6 +45,7 @@ import org.apache.logging.log4j.core.config.builder.api.LoggerComponentBuilder;
 import org.apache.logging.log4j.core.config.builder.api.RootLoggerComponentBuilder;
 import org.apache.logging.log4j.core.config.builder.api.ScriptComponentBuilder;
 import org.apache.logging.log4j.core.config.builder.api.ScriptFileComponentBuilder;
+import org.apache.logging.log4j.core.util.Throwables;
 
 /**
  * @param <T> The BuiltConfiguration type.
@@ -44,6 +53,9 @@ import org.apache.logging.log4j.core.config.builder.api.ScriptFileComponentBuild
  */
 public class DefaultConfigurationBuilder<T extends BuiltConfiguration> implements ConfigurationBuilder<T> {
 
+    private static final String INDENT = "  ";
+    private static final String EOL = System.lineSeparator();
+    
     private final Component root = new Component();
     private Component loggers;
     private Component appenders;
@@ -53,14 +65,15 @@ public class DefaultConfigurationBuilder<T extends BuiltConfiguration> implement
     private Component scripts;
     private final Class<T> clazz;
     private ConfigurationSource source;
-    private int monitorInterval = 0;
-    private Level level = null;
-    private String verbosity = null;
-    private String packages = null;
-    private String shutdownFlag = null;
-    private String advertiser = null;
-
-    private String name = null;
+    private int monitorInterval;
+    private Level level;
+    private String verbosity;
+    private String destination;
+    private String packages;
+    private String shutdownFlag;
+    private String advertiser;
+    private LoggerContext loggerContext;
+    private String name;
 
     @SuppressWarnings("unchecked")
     public DefaultConfigurationBuilder() {
@@ -151,8 +164,8 @@ public class DefaultConfigurationBuilder<T extends BuiltConfiguration> implement
             if (source == null) {
                 source = ConfigurationSource.NULL_SOURCE;
             }
-            final Constructor<T> constructor = clazz.getConstructor(ConfigurationSource.class, Component.class);
-            configuration = constructor.newInstance(source, root);
+            final Constructor<T> constructor = clazz.getConstructor(LoggerContext.class, ConfigurationSource.class, Component.class);
+            configuration = constructor.newInstance(loggerContext, source, root);
             configuration.setMonitorInterval(monitorInterval);
             configuration.getRootNode().getAttributes().putAll(root.getAttributes());
             if (name != null) {
@@ -164,6 +177,9 @@ public class DefaultConfigurationBuilder<T extends BuiltConfiguration> implement
             if (verbosity != null) {
                 configuration.getStatusConfiguration().withVerbosity(verbosity);
             }
+            if (destination != null) {
+                configuration.getStatusConfiguration().withDestination(destination);
+            }
             if (packages != null) {
                 configuration.setPluginPackages(packages);
             }
@@ -183,6 +199,126 @@ public class DefaultConfigurationBuilder<T extends BuiltConfiguration> implement
         return configuration;
     }
 
+    @Override
+    public void writeXmlConfiguration(final OutputStream output) throws IOException {
+        try {
+            final XMLStreamWriter xmlWriter = XMLOutputFactory.newInstance().createXMLStreamWriter(output);
+            writeXmlConfiguration(xmlWriter);
+            xmlWriter.close();
+        } catch (final XMLStreamException e) {
+            if (e.getNestedException() instanceof IOException) {
+                throw (IOException)e.getNestedException();
+            }
+            Throwables.rethrow(e);
+        }
+    }
+
+    @Override
+    public String toXmlConfiguration() {
+        final StringWriter sw = new StringWriter();
+        try {
+            final XMLStreamWriter xmlWriter = XMLOutputFactory.newInstance().createXMLStreamWriter(sw);
+            writeXmlConfiguration(xmlWriter);
+            xmlWriter.close();
+        } catch (final XMLStreamException e) {
+            Throwables.rethrow(e);
+        }
+        return sw.toString();
+    }
+
+    private void writeXmlConfiguration(final XMLStreamWriter xmlWriter) throws XMLStreamException {
+        xmlWriter.writeStartDocument();
+        xmlWriter.writeCharacters(EOL);
+
+        xmlWriter.writeStartElement("Configuration");
+        if (name != null) {
+            xmlWriter.writeAttribute("name", name);
+        }
+        if (level != null) {
+            xmlWriter.writeAttribute("status", level.name());
+        }
+        if (verbosity != null) {
+            xmlWriter.writeAttribute("verbose", verbosity);
+        }
+        if (destination != null) {
+            xmlWriter.writeAttribute("dest", destination);
+        }
+        if (packages != null) {
+            xmlWriter.writeAttribute("packages", packages);
+        }
+        if (shutdownFlag != null) {
+            xmlWriter.writeAttribute("shutdownHook", shutdownFlag);
+        }
+        if (advertiser != null) {
+            xmlWriter.writeAttribute("advertiser", advertiser);
+        }
+        if (monitorInterval > 0) {
+            xmlWriter.writeAttribute("monitorInterval", String.valueOf(monitorInterval));
+        }
+
+        xmlWriter.writeCharacters(EOL);
+
+        writeXmlSection(xmlWriter, properties);
+        writeXmlSection(xmlWriter, scripts);
+        writeXmlSection(xmlWriter, customLevels);
+        if (filters.getComponents().size() == 1) {
+            writeXmlComponent(xmlWriter, filters.getComponents().get(0), 1);
+        } else if (filters.getComponents().size() > 1) {
+            writeXmlSection(xmlWriter, filters);
+        }
+        writeXmlSection(xmlWriter, appenders);
+        writeXmlSection(xmlWriter, loggers);
+
+        xmlWriter.writeEndElement(); // "Configuration"
+        xmlWriter.writeCharacters(EOL);
+
+        xmlWriter.writeEndDocument();
+    }
+
+    private void writeXmlSection(final XMLStreamWriter xmlWriter, final Component component) throws XMLStreamException {
+        if (!component.getAttributes().isEmpty() || !component.getComponents().isEmpty() || component.getValue() != null) {
+            writeXmlComponent(xmlWriter, component, 1);
+        }
+    }
+
+    private void writeXmlComponent(final XMLStreamWriter xmlWriter, final Component component, final int nesting) throws XMLStreamException {
+        if (!component.getComponents().isEmpty() || component.getValue() != null) {
+            writeXmlIndent(xmlWriter, nesting);
+            xmlWriter.writeStartElement(component.getPluginType());
+            writeXmlAttributes(xmlWriter, component);
+            if (!component.getComponents().isEmpty()) {
+                xmlWriter.writeCharacters(EOL);
+            }
+            for (final Component subComponent : component.getComponents()) {
+                writeXmlComponent(xmlWriter, subComponent, nesting + 1);
+            }
+            if (component.getValue() != null) {
+                xmlWriter.writeCharacters(component.getValue());
+            }
+            if (!component.getComponents().isEmpty()) {
+                writeXmlIndent(xmlWriter, nesting);
+            }
+            xmlWriter.writeEndElement();
+        } else {
+            writeXmlIndent(xmlWriter, nesting);
+            xmlWriter.writeEmptyElement(component.getPluginType());
+            writeXmlAttributes(xmlWriter, component);
+        }
+        xmlWriter.writeCharacters(EOL);
+    }
+
+    private void writeXmlIndent(final XMLStreamWriter xmlWriter, final int nesting) throws XMLStreamException {
+        for (int i = 0; i < nesting; i++) {
+            xmlWriter.writeCharacters(INDENT);
+        }
+    }
+
+    private void writeXmlAttributes(final XMLStreamWriter xmlWriter, final Component component) throws XMLStreamException {
+        for (final Map.Entry<String, String> attribute : component.getAttributes().entrySet()) {
+            xmlWriter.writeAttribute(attribute.getKey(), attribute.getValue());
+        }
+    }
+
     @Override
     public ScriptComponentBuilder newScript(final String name, final String language, final String text) {
         return new DefaultScriptComponentBuilder(this, name, language, text);
@@ -211,7 +347,7 @@ public class DefaultConfigurationBuilder<T extends BuiltConfiguration> implement
 
     @Override
     public LoggerComponentBuilder newAsyncLogger(final String name, final Level level) {
-        return new DefaultLoggerComponentBuilder(this, name, level.toString(), "AsyncLogger", false);
+        return new DefaultLoggerComponentBuilder(this, name, level.toString(), "AsyncLogger");
     }
 
     @Override
@@ -221,7 +357,7 @@ public class DefaultConfigurationBuilder<T extends BuiltConfiguration> implement
 
     @Override
     public LoggerComponentBuilder newAsyncLogger(final String name, final String level) {
-        return new DefaultLoggerComponentBuilder(this, name, level, "AsyncLogger", false);
+        return new DefaultLoggerComponentBuilder(this, name, level, "AsyncLogger");
     }
 
     @Override
@@ -231,7 +367,7 @@ public class DefaultConfigurationBuilder<T extends BuiltConfiguration> implement
 
     @Override
     public RootLoggerComponentBuilder newAsyncRootLogger(final Level level) {
-        return new DefaultRootLoggerComponentBuilder(this, level.toString(), "AsyncRoot", false);
+        return new DefaultRootLoggerComponentBuilder(this, level.toString(), "AsyncRoot");
     }
 
     @Override
@@ -241,7 +377,7 @@ public class DefaultConfigurationBuilder<T extends BuiltConfiguration> implement
 
     @Override
     public RootLoggerComponentBuilder newAsyncRootLogger(final String level) {
-        return new DefaultRootLoggerComponentBuilder(this, level, "AsyncRoot", false);
+        return new DefaultRootLoggerComponentBuilder(this, level, "AsyncRoot");
     }
 
     @Override
@@ -291,7 +427,7 @@ public class DefaultConfigurationBuilder<T extends BuiltConfiguration> implement
 
     @Override
     public LoggerComponentBuilder newLogger(final String name, final Level level) {
-        return new DefaultLoggerComponentBuilder(this, name, level.toString(), true);
+        return new DefaultLoggerComponentBuilder(this, name, level.toString());
     }
 
     @Override
@@ -301,7 +437,7 @@ public class DefaultConfigurationBuilder<T extends BuiltConfiguration> implement
 
     @Override
     public LoggerComponentBuilder newLogger(final String name, final String level) {
-        return new DefaultLoggerComponentBuilder(this, name, level, true);
+        return new DefaultLoggerComponentBuilder(this, name, level);
     }
 
     @Override
@@ -311,7 +447,7 @@ public class DefaultConfigurationBuilder<T extends BuiltConfiguration> implement
 
     @Override
     public RootLoggerComponentBuilder newRootLogger(final Level level) {
-        return new DefaultRootLoggerComponentBuilder(this, level.toString(), true);
+        return new DefaultRootLoggerComponentBuilder(this, level.toString());
     }
 
     @Override
@@ -321,7 +457,7 @@ public class DefaultConfigurationBuilder<T extends BuiltConfiguration> implement
 
     @Override
     public RootLoggerComponentBuilder newRootLogger(final String level) {
-        return new DefaultRootLoggerComponentBuilder(this, level, true);
+        return new DefaultRootLoggerComponentBuilder(this, level);
     }
 
     @Override
@@ -389,9 +525,21 @@ public class DefaultConfigurationBuilder<T extends BuiltConfiguration> implement
         return this;
     }
 
+    @Override
+    public ConfigurationBuilder<T> setDestination(final String destination) {
+        this.destination = destination;
+        return this;
+    }
+
+    @Override
+    public void setLoggerContext(final LoggerContext loggerContext) {
+        this.loggerContext = loggerContext;
+    }
+
     @Override
     public ConfigurationBuilder<T> addRootProperty(final String key, final String value) {
         root.getAttributes().put(key, value);
         return this;
     }
+
 }
