@@ -34,7 +34,14 @@ public class Log4jXmlObjectMapper extends XmlMapper {
      * Create a new instance using the {@link Log4jXmlModule}.
      */
     public Log4jXmlObjectMapper() {
-        super(new Log4jXmlModule());
+        this(true);
+    }
+
+    /**
+     * Create a new instance using the {@link Log4jXmlModule}.
+     */
+    public Log4jXmlObjectMapper(final boolean includeStacktrace) {
+        super(new Log4jXmlModule(includeStacktrace));
         this.setSerializationInclusion(JsonInclude.Include.NON_EMPTY);
     }
 
