@@ -43,20 +43,24 @@ import org.apache.logging.log4j.MarkerManager;
 import org.apache.logging.log4j.ThreadContext;
 import org.apache.logging.log4j.core.LogEvent;
 import org.apache.logging.log4j.core.appender.AppenderLoggingException;
+import org.apache.logging.log4j.junit.ThreadContextStackRule;
 import org.apache.logging.log4j.message.Message;
 import org.easymock.Capture;
 import org.easymock.EasyMock;
 import org.easymock.IAnswer;
 import org.junit.After;
 import org.junit.Before;
+import org.junit.Rule;
 import org.junit.Test;
 
 public class NoSqlDatabaseManagerTest {
     private NoSqlConnection<Map<String, Object>, DefaultNoSqlObject> connection;
     private NoSqlProvider<NoSqlConnection<Map<String, Object>, DefaultNoSqlObject>> provider;
 
+    @Rule
+    public final ThreadContextStackRule threadContextRule = new ThreadContextStackRule(); 
+
     @Before
-    @SuppressWarnings("unchecked")
     public void setUp() {
         this.provider = createStrictMock(NoSqlProvider.class);
         this.connection = createStrictMock(NoSqlConnection.class);
@@ -68,59 +72,47 @@ public class NoSqlDatabaseManagerTest {
     }
 
     @Test
-    public void testConnection() {
-        replay(this.provider, this.connection);
+	public void testConnection() {
+		replay(this.provider, this.connection);
 
-        final NoSqlDatabaseManager<?> manager = NoSqlDatabaseManager.getNoSqlDatabaseManager("name", 0, this.provider);
+		try (final NoSqlDatabaseManager<?> manager = NoSqlDatabaseManager.getNoSqlDatabaseManager("name", 0,
+				this.provider)) {
 
-        assertNotNull("The manager should not be null.", manager);
+			assertNotNull("The manager should not be null.", manager);
 
-        try {
-            verify(this.provider, this.connection);
-            reset(this.provider, this.connection);
-            expect(this.provider.getConnection()).andReturn(this.connection);
-            replay(this.provider, this.connection);
+			verify(this.provider, this.connection);
+			reset(this.provider, this.connection);
+			expect(this.provider.getConnection()).andReturn(this.connection);
+			replay(this.provider, this.connection);
 
-            manager.connectAndStart();
-            manager.commitAndClose();
-        } finally {
-            try {
-                manager.release();
-            } catch (final Throwable ignore) {
-                /* */
-            }
-        }
-    }
+			manager.connectAndStart();
+			manager.commitAndClose();
+		}
+	}
 
     @Test
-    public void testWriteInternalNotConnected01() {
-        replay(this.provider, this.connection);
+	public void testWriteInternalNotConnected01() {
+		replay(this.provider, this.connection);
 
-        final NoSqlDatabaseManager<?> manager = NoSqlDatabaseManager.getNoSqlDatabaseManager("name", 0, this.provider);
+		try (final NoSqlDatabaseManager<?> manager = NoSqlDatabaseManager.getNoSqlDatabaseManager("name", 0,
+				this.provider)) {
 
-        try {
-            verify(this.provider, this.connection);
-            reset(this.provider, this.connection);
+			verify(this.provider, this.connection);
+			reset(this.provider, this.connection);
 
-            final LogEvent event = createStrictMock(LogEvent.class);
-            replay(this.provider, this.connection, event);
+			final LogEvent event = createStrictMock(LogEvent.class);
+			replay(this.provider, this.connection, event);
 
-            try {
-                manager.writeInternal(event);
-                fail("Expected AppenderLoggingException.");
-            } catch (final AppenderLoggingException ignore) {
-                /* */
-            }
+			try {
+				manager.writeInternal(event);
+				fail("Expected AppenderLoggingException.");
+			} catch (final AppenderLoggingException ignore) {
+				/* */
+			}
 
-            verify(event);
-        } finally {
-            try {
-                manager.release();
-            } catch (final Throwable ignore) {
-                /* */
-            }
-        }
-    }
+			verify(event);
+		}
+	}
 
     @Test
     public void testWriteInternalNotConnected02() {
@@ -153,7 +145,7 @@ public class NoSqlDatabaseManagerTest {
             }
         } finally {
             try {
-                manager.release();
+                manager.close();
             } catch (final Throwable ignore) {
                 /* */
             }
@@ -242,7 +234,7 @@ public class NoSqlDatabaseManagerTest {
             verify(this.provider, this.connection, event, message);
         } finally {
             try {
-                manager.release();
+                manager.close();
             } catch (final Throwable ignore) {
                 /* */
             }
@@ -379,7 +371,7 @@ public class NoSqlDatabaseManagerTest {
             verify(this.provider, this.connection, event, message);
         } finally {
             try {
-                manager.release();
+                manager.close();
             } catch (final Throwable ignore) {
                 /* */
             }
@@ -578,7 +570,7 @@ public class NoSqlDatabaseManagerTest {
             verify(this.provider, this.connection, event, message);
         } finally {
             try {
-                manager.release();
+                manager.close();
             } catch (final Throwable ignore) {
                 /* */
             }
